@@ -1,6 +1,10 @@
 import { sortBy } from 'lodash';
 
-import { UrlQueryMap, Labels, DataSourceInstanceSettings, DataSourceJsonData } from '@grafana/data';
+import { UrlQueryMap, Labels } from '@grafana/data';
+import { GrafanaEdition } from '@grafana/data/src/types/config';
+import { config, isFetchError } from '@grafana/runtime';
+import { DataSourceRef } from '@grafana/schema';
+import { escapePathSeparators } from 'app/features/alerting/unified/utils/rule-id';
 import { alertInstanceKey } from 'app/features/alerting/unified/utils/rules';
 import { SortOrder } from 'app/plugins/panel/alertlist/types';
 import { Alert, CombinedRule, FilterState, RulesSource, SilenceFilterState } from 'app/types/unified-alerting';
@@ -25,19 +29,36 @@ export function createViewLink(ruleSource: RulesSource, rule: CombinedRule, retu
   return createUrl(`/alerting/${paramSource}/${paramId}/view`, { returnTo });
 }
 
-export function createExploreLink(dataSourceName: string, query: string) {
+export function createExploreLink(datasource: DataSourceRef, query: string) {
+  const { uid, type } = datasource;
+
   return createUrl(`/explore`, {
     left: JSON.stringify({
-      datasource: dataSourceName,
-      queries: [{ refId: 'A', datasource: dataSourceName, expr: query }],
+      datasource: datasource.uid,
+      queries: [{ refId: 'A', datasource: { uid, type }, expr: query }],
       range: { from: 'now-1h', to: 'now' },
     }),
   });
 }
 
+export function createContactPointLink(contactPoint: string, alertManagerSourceName = ''): string {
+  return createUrl(`/alerting/notifications/receivers/${encodeURIComponent(contactPoint)}/edit`, {
+    alertmanager: alertManagerSourceName,
+  });
+}
+
+export function createMuteTimingLink(muteTimingName: string, alertManagerSourceName = ''): string {
+  return createUrl('/alerting/routes/mute-timing/edit', {
+    muteName: muteTimingName,
+    alertmanager: alertManagerSourceName,
+  });
+}
+
 export function createShareLink(ruleSource: RulesSource, rule: CombinedRule): string {
   if (isCloudRulesSource(ruleSource)) {
-    return createAbsoluteUrl(`/alerting/${encodeURIComponent(ruleSource.name)}/${encodeURIComponent(rule.name)}/find`);
+    return createAbsoluteUrl(
+      `/alerting/${encodeURIComponent(ruleSource.name)}/${encodeURIComponent(escapePathSeparators(rule.name))}/find`
+    );
   }
 
   return window.location.href.split('?')[0];
@@ -77,15 +98,25 @@ export function recordToArray(record: Record<string, string>): Array<{ key: stri
   return Object.entries(record).map(([key, value]) => ({ key, value }));
 }
 
-export function makeAMLink(path: string, alertManagerName?: string, options?: Record<string, string>): string {
+type URLParamsLike = ConstructorParameters<typeof URLSearchParams>[0];
+export function makeAMLink(path: string, alertManagerName?: string, options?: URLParamsLike): string {
   const search = new URLSearchParams(options);
+
   if (alertManagerName) {
-    search.append(ALERTMANAGER_NAME_QUERY_KEY, alertManagerName);
+    search.set(ALERTMANAGER_NAME_QUERY_KEY, alertManagerName);
   }
   return `${path}?${search.toString()}`;
 }
 
+export const escapeQuotes = (input: string) => input.replace(/\"/g, '\\"');
+
+export function wrapWithQuotes(input: string) {
+  const alreadyWrapped = input.startsWith('"') && input.endsWith('"');
+  return alreadyWrapped ? escapeQuotes(input) : `"${escapeQuotes(input)}"`;
+}
+
 export function makeRuleBasedSilenceLink(alertManagerSourceName: string, rule: CombinedRule) {
+  // we wrap the name of the alert with quotes since it might contain starting and trailing spaces
   const labels: Labels = {
     alertname: rule.name,
     ...rule.labels,
@@ -104,14 +135,42 @@ export function makeLabelBasedSilenceLink(alertManagerSourceName: string, labels
   return createUrl('/alerting/silence/new', silenceUrlParams);
 }
 
-export function makeDataSourceLink<T extends DataSourceJsonData>(dataSource: DataSourceInstanceSettings<T>) {
-  return createUrl(`/datasources/edit/${dataSource.uid}`);
+export function makeDataSourceLink(uid: string) {
+  return createUrl(`/datasources/edit/${uid}`);
 }
 
 export function makeFolderLink(folderUID: string): string {
   return createUrl(`/dashboards/f/${folderUID}`);
 }
 
+export function makeFolderAlertsLink(folderUID: string, title: string): string {
+  return createUrl(`/dashboards/f/${folderUID}/${title}/alerting`);
+}
+
+export function makeFolderSettingsLink(uid: string): string {
+  return createUrl(`/dashboards/f/${uid}/settings`);
+}
+
+export function makeDashboardLink(dashboardUID: string): string {
+  return createUrl(`/d/${encodeURIComponent(dashboardUID)}`);
+}
+
+export function makePanelLink(
+  dashboardUID: string,
+  panelId: string,
+  mode: 'view' | 'edit' = 'view',
+  tab?: 'alert' | 'transform' | 'query'
+): string {
+  const panelIdParam = mode === 'edit' ? 'editPanel' : 'viewPanel';
+  const panelParams = new URLSearchParams({ [panelIdParam]: panelId });
+
+  if (tab) {
+    panelParams.set('tab', tab);
+  }
+
+  return createUrl(`/d/${encodeURIComponent(dashboardUID)}`, panelParams);
+}
+
 // keep retrying fn if it's error passes shouldRetry(error) and timeout has not elapsed yet
 export function retryWhile<T, E = Error>(
   fn: () => Promise<T>,
@@ -166,3 +225,26 @@ export function sortAlerts(sortOrder: SortOrder, alerts: Alert[]): Alert[] {
 
   return result;
 }
+
+export function isOpenSourceEdition() {
+  const buildInfo = config.buildInfo;
+  return buildInfo.edition === GrafanaEdition.OpenSource;
+}
+
+export function isLocalDevEnv() {
+  const buildInfo = config.buildInfo;
+  return buildInfo.env === 'development';
+}
+
+export function isErrorLike(error: unknown): error is Error {
+  return 'message' in (error as Error);
+}
+
+export function stringifyErrorLike(error: unknown): string {
+  const fetchError = isFetchError(error);
+  if (fetchError) {
+    return error.data.message;
+  }
+
+  return isErrorLike(error) ? error.message : String(error);
+}
